package gs.test.v0.definition.pos

import gs.test.v0.IOSuite
import gs.test.v0.definition.*
import munit.*

/** These tests are sensitive to changes, even in formatting! They are looking
  * for specific line numbers in this source code, so any sort of newline that
  * occurs prior to the sub-test-definitions will cause the checked positions to
  * change!
  */
class SourcePositionTests extends IOSuite:

  import SourcePositionTests.*

  test("should provide the source position of a failed check") {
    lookForSourcePosition(new G1, 46)
  }

  test("should provide the source position of an explicit failure") {
    lookForSourcePosition(new G2, 55)
  }

  private def lookForSourcePosition(
    groupDef: TestGroup.IO,
    line: Int
  ): Unit =
    val group = groupDef.compile()
    group.tests match
      case t1 :: Nil =>
        t1.unitOfWork.value.unsafeRunSync() match
          case Left(TestFailure.AssertionFailed(_, _, _, pos)) =>
            assertEquals(pos.file.endsWith(SourceFileName), true)
            assertEquals(pos.line, line)
          case Left(TestFailure.TestRequestedFailure(_, pos)) =>
            assertEquals(pos.file.endsWith(SourceFileName), true)
            assertEquals(pos.line, line)
          case _ => fail("Sub-test did not fail as expected.")
      case _ =>
        fail("Wrong number of tests - position tests need one test per group.")

object SourcePositionTests:

  class G1 extends TestGroup.IO:
    override def name: String = "G1"

    test(pid"t1", "pos").pure {
      check(false).isTrue()
    }

  end G1

  class G2 extends TestGroup.IO:
    override def name: String = "G2"

    test(pid"t2", "pos").pure {
      gs.test.v0.definition.fail("Expected Failure")
    }

  end G2

  val SourceFileName: String =
    "modules/api-definition/src/test/scala/gs/test/v0/definition/pos/SourcePositionTests.scala"

end SourcePositionTests
